// This macro is an auxiliary tool used to convert ROI positions between two MRI datasets (2D and 3D, proton density weighted)
// which have the same number of slices (39), but different slice thicknesses (360 um vs. 180 um) 

// (c) 2022 Sinisa Skokic, Laboratory for Regenerative Neuroscience, University of Zagreb School of Medicine
// contact: <name>.<surname>@hiim.hr

function leftPad(n, width) {
	// serves to add leading zeros to ROI numbers when renaming
	s =""+n;
  	while (lengthOf(s)<width)
   	   s = "0"+s;
  	return s;
}

// initial cleanup
waitForUser("WARNING!\n \nThis macro will close all open images, delete all ROIs in ROI manager\n"
	+"and clear everything in the Results table!\n "
	+"\nPress OK to proceed.");

run("Clear Results");   // clear previous results in Results table
print("\\Clear");   // clear log window
if (roiManager("count") > 0) {
	roiManager("deselect"); roiManager("delete");
}
run("Select None");
close("*");

// load input files
path_ROI = File.openDialog("Load ROI set of PDw image");
dir_ROI = File.getParent(path_ROI);
fileName_ROI = File.getName(path_ROI);

path_PDw = File.openDialog("Load PDw image");
dir_PDw = File.getParent(path_PDw);
fileName_PDw = File.getName(path_PDw);

path_PDw3D = File.openDialog("Load PDw-3D image");
dir_PDw3D = File.getParent(path_PDw3D);
fileName_PDw3D = File.getName(path_PDw3D);

saveDir =   getDirectory("select destination folder for output files.");


// open all selected files
open(path_ROI); // open the ROIset file
nROI = roiManager("count");
// 
open(path_PDw); // open PDw file
run("Rotate 90 Degrees Right");
image_PDw = getImageID();
title_PDw = getTitle();
//
open(path_PDw3D); // open the T2map file
run("Rotate 90 Degrees Right");
image_PDw3D = getImageID();
title_PDw3D = getTitle();

selectImage(image_PDw);

indexes = newArray(nROI);
for(i = 0; i<nROI; i++) {
	indexes[i] = i;
	roiManager("select", i);
	roiNameOriginal = Roi.getName;
	Roi.getPosition(ch, sl, fr);
	new_slice = 2*sl - 19;  // 19 is half of the ROIset length
	print("ROI " + roiNameOriginal + " - new slice number: "+new_slice);
	run("Select None");
	setSlice(new_slice);
	run("Restore Selection");
	Roi.setPosition(0, new_slice, 0);
	roiManager("add");
	roiManager("select", nROI + i);
	//roiManager("rename", roiNameOriginal+" moved to new slice #"+leftPad(new_slice, 2));
	//roiManager("rename", "ROI "+roiNameOriginal+" moved to new slice #"+leftPad(new_slice, 2));
	//waitForUser;
}


roiManager("select", indexes);
roiManager("delete");  // delete original ROIs from ROI manager
// interpolate between new ROIs
roiManager("select", indexes); 
roiManager("Sort");
roiManager("Interpolate ROIs");

for (i=0; i<roiManager("count");i++) {
	roiManager("select", i);
	run("Interpolate", "interval=0.5 smooth");  // smoothen new ROIs
	roiManager("update");
}

// save new ROIset adapted to PDw-3D image
fileName_ROIset = substring(fileName_ROI, 0, fileName_ROI.length - 4) + " adapted to PDw-3D image.zip";
roiManager("deselect");
roiManager("save", saveDir+fileName_ROIset);

selectImage(image_PDw); close(); 

