// This macro is used as a tool to manually align the selected uCT image and MRI image, 
// picked in "adapt_CTroi_to_MRI.ijm" macro. 
//
// The MRI image can be loaded as "bottom" image, and uCT image as "top" image.
// User can use sliders to perform fine translation and rotation adjustments
// until the best visual match is obtained (minumum number of bright pixels in the bone area).
//
// (c) 2022 Sinisa Skokic, Laboratory for Regenerative Neuroscience, University of Zagreb School of Medicine
// contact: <name>.<surname>@hiim.hr


scr_wdth = screenWidth;
scr_hght = screenHeight;

// load input file if no image is open 
/*
	if (nImages > 0) {
		img_titles = getList("image.titles");
		selectImage(img_titles[nImages-1]); // select last opened image if any are open
		img_im1 = getImageID();
		title_im1 = getTitle();
		getVoxelSize(px_w_im1, px_h_im1, px_d_im1, px_unit_im1);
		wdth_im1 = getValue("Width")/px_w_im1;
		hght_im1 = getValue("Height")/px_h_im1;
		print("PDw image width = "+wdth_im1+"; image height = "+hght_im1+"; voxel size [mm]= "+px_w_im1+" / "+px_h_im1+" / "+px_d_im1);
	}
	else {
		path_im1 = File.openDialog("Load bottom image");
		dir_im1 = File.getParent(path_im1);
		fileName_im1 = File.getName(path_im1);
		// get image info 
		open(path_im1); // open bottom image file
		image_im1 = getImageID();
		title_im1 = getTitle();
		getVoxelSize(px_w_im1, px_h_im1, px_d_im1, px_unit_im1);
		wdth_im1 = getValue("Width")/px_w_im1;
		hght_im1 = getValue("Height")/px_h_im1;
		print("PDw image width = "+wdth_im1+"; image height = "+hght_im1+"; voxel size [mm]= "+px_w_im1+" / "+px_h_im1+" / "+px_d_im1);
	}

  */

// load input files
path_im1 = File.openDialog("Load bottom image");
dir_im1 = File.getParent(path_im1);
fileName_im1 = File.getName(path_im1);
// get image info 
open(path_im1); // open bottom image file
image_im1 = getImageID();
title_im1 = getTitle();
getVoxelSize(px_w_im1, px_h_im1, px_d_im1, px_unit_im1);
wdth_im1 = getValue("Width")/px_w_im1;
hght_im1 = getValue("Height")/px_h_im1;
run("8-bit");
print("bottom image: width = "+wdth_im1+"; height = "+hght_im1+"; voxel [mm]= "+px_w_im1+" / "+px_h_im1+" / "+px_d_im1);


path_im2 = File.openDialog("Load top image");
dir_im2 = File.getParent(path_im2);
fileName_im2 = File.getName(path_im2);
// get image info 
open(path_im2); // open top image file
image_im2 = getImageID();
title_im2 = getTitle();
getVoxelSize(px_w_im2, px_h_im2 px_d_im2, px_unit_im2);
wdth_im2 = getValue("Width")/px_w_im2;
hght_im2 = getValue("Height")/px_h_im2;
run("8-bit");
print("top image: width = "+wdth_im1+"; height = "+hght_im1+"; voxel [mm]= "+px_w_im1+" / "+px_h_im1+" / "+px_d_im1);



///// prepare for aligning....

repeat = 1; 
rot_dft = 0; trX_dft = 0; trY_dft = 0; op_dft = 0.3;
// calculate canvas size for the stack (slightly larger than the bigger of the two loaded images)
  canvas_X = round(1.1*maxOf(wdth_im1, wdth_im2));
  canvas_Y = round(1.1*maxOf(hght_im1, hght_im2));
  

// create initial duplicates of both images and adjust canvas size (avoid errors when creating stack)
// keep alignment fixed at top-left corner to preserve distances for alignment of sagittal uCT slices
selectImage(image_im1);
run("Duplicate...", "duplicate");  // create a fresh copy for manipulation
image_im1copy = getImageID();
title_im1copy = getTitle();
run("Canvas Size...", "width="+canvas_X+" height="+canvas_Y+" position=Top-Left zero");  // expand canvas to have enough room for rotation
//
selectImage(image_im2);
run("Duplicate...", "duplicate");  // create a fresh copy for manipulation
image_im2copy = getImageID();
title_im2copy = getTitle();
run("Canvas Size...", "width="+canvas_X+" height="+canvas_Y+" position=Top-Left zero");  // expand canvas to have enough room for rotation


// create stack for aligning
// calculate size, create fixed size canvas and put it always on the same position
run("Concatenate...", "  title=ALIGNMENT_STACK keep image1=["+title_im1copy+"] image2=["+title_im2copy+"] image3=[-- None --]");
image_stck = getImageID();
run("Z Project...", "projection=[Sum Slices]");
image_Zstck = getImageID();
selectImage(image_stck); close();
max_magnification = round(minOf(scr_wdth/canvas_X, scr_hght/canvas_Y));
print("Max magnification = "+max_magnification);
extra_scr_wdth = scr_wdth - round(max_magnification/2)*canvas_X;
extra_scr_hght = scr_hght - round(max_magnification/2)*canvas_Y;
selectImage(image_Zstck);
setLocation(round(extra_scr_wdth/2), round(extra_scr_hght/2),
   round(max_magnification/2)*canvas_X, round(max_magnification/2)*canvas_Y);



//// aligning

while (repeat == 1) {
	dialog_title = "Transform and align images";
	Dialog.create("Transform and align images"); 
	Dialog.addSlider("Rotate [deg]", -180, 180, rot_dft);
	Dialog.addSlider("Translate X [px]", -100, 100, trX_dft);
	Dialog.addSlider("Translate Y [px]", -100, 100, trY_dft);
	Dialog.addSlider("Overlay opacity", 0, 1.0, op_dft);
	opt = newArray("Preview", "Save and Exit");
	Dialog.addRadioButtonGroup("Finalize?", opt, 1, 2, opt[0]);
	
	Dialog.setLocation(screenWidth/8, screenHeight/8);
	Dialog.show();
	
	rot = Dialog.getNumber();
	trX = Dialog.getNumber();
	trY = Dialog.getNumber();
	op = Dialog.getNumber();
	button = Dialog.getRadioButton(); 
	if (button == "Save and Exit") {
		repeat = 0;
	}

	rot_dft = rot;
	trX_dft = trX;
	trY_dft = trY;
	op_dft = op;

	selectImage(image_Zstck); close();

	run("Concatenate...", "  title=ALIGNMENT_STACK keep image1=["+title_im1copy+"] image2=["+title_im2copy+"] image3=[-- None --]");
   	// apply alignment with set values	
	setSlice(2);
	run("Rotate... ", "angle="+rot+" grid=0 interpolation=Bilinear fill slice");  
	run("Translate...", "x="+trX+" y="+trY+" interpolation=None slice");
	image_stck = getImageID();
	run("Z Project...", "projection=[Sum Slices]");
	image_Zstck = getImageID();
	selectImage(image_stck); close();
	selectImage(image_Zstck); 
	setLocation(round(extra_scr_wdth/2), round(extra_scr_hght/2),
   		round(max_magnification/2)*canvas_X, round(max_magnification/2)*canvas_Y);



}

close("\\Others");
